package cn.com.qmth.stmms.biz.exam.dao;

import java.util.Date;
import java.util.List;

import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.PagingAndSortingRepository;

import cn.com.qmth.stmms.biz.exam.model.ExamStudent;

public interface ExamStudentDao
        extends PagingAndSortingRepository<ExamStudent, Integer>, JpaSpecificationExecutor<ExamStudent> {

    public List<ExamStudent> findByExamId(int examId, Pageable pageable);

    public List<ExamStudent> findByExamIdAndUploadAndAbsent(int examId, boolean upload, boolean absent,
            Pageable pageable);

    public List<ExamStudent> findByExamIdAndCampusName(int examId, String campusName, Pageable pageable);

    public List<ExamStudent> findByExamIdAndExamNumber(int examId, String examNumber);

    @Query("select count(s) from ExamStudent s where s.examId=?1 and s.campusName=?2")
    public long countByExamIdAndCampusName(int examId, String campusName);

    @Query("select count(distinct s.campusName) from ExamStudent s where s.examId=?1")
    public long countCampusNameByExamId(int examId);

    @Query("select s from ExamStudent s where s.examId=?1 group by s.subjectCode")
    public List<ExamStudent> findDistinctSubjectByExamId(int examId, Pageable pageable);

    @Query("select s from ExamStudent s where s.examId=?1 and s.campusName=?2 group by s.subjectCode")
    public List<ExamStudent> findDistinctSubjectByExamIdAndCampusName(int examId, String campusName);

    @Query("select s.campusName from ExamStudent s where s.examId=?1 group by s.campusName")
    public List<String> findDistinctCampusName(int examId);

    @Query("select s from ExamStudent s where s.examId=?1 group by s.campusName")
    public List<ExamStudent> findDistinctCampusName(int examId, Pageable pageable);

    @Query("select count(c) from Campus c where exists (select s.id from ExamStudent s where "
            + "c.schoolId=s.schoolId and c.name=s.campusName and s.examId=?1)")
    public long countDistinctCampusName(int examId);

    @Query("select s.packageCode from ExamStudent s where s.examId=?1 group by s.packageCode")
    public List<String> findDistinctPackageCode(int examId);

    @Modifying
    @Query("update ExamStudent s set s.subjectiveScore=?2, s.subjectiveScoreList=?3 where s.id=?1")
    public void updateSubjectiveScore(int id, double score, String scoreList);

    @Modifying
    @Query("update ExamStudent s set s.subjectName=?3, s.subjectLevel=?4, s.subjectCategory=?5 where s.examId=?1 and s.subjectCode=?2")
    public void updateSubjectInfo(int examId, String subjectCode, String subjectName, String subjectLevel,
            String subjectCategory);

    @Modifying
    @Query("update ExamStudent s set s.exception=?2 where s.id=?1")
    public void updateException(int id, boolean exception);

    @Modifying
    @Query("delete from ExamStudent s where s.examId=?1")
    public void deleteByExamId(int examId);

    @Modifying
    @Query("delete from ExamStudent s where s.examId=?1 and s.campusName=?2")
    public void deleteByExamIdAndCampusName(int examId, String campusName);

    public ExamStudent findByExamIdAndSubjectCodeAndStudentCode(Integer id, String subjectCode, String studentCode);

    public ExamStudent findBySchoolIdAndSubjectCodeAndStudentCodeAndRemark(Integer schoolId, String subjectCode,
            String studentCode, String examSeqCode);

    public List<ExamStudent> findByExamIdAndUploadTimeAfter(int examId, Date date);

    public List<ExamStudent> findByExamIdAndSubjectCodeAndUploadTimeAfter(int examId, String code, Date date,
            Pageable page);

    @Query("select count(s) from ExamStudent s where s.examId=?1 and s.subjectiveScoreList != null ")
    public long countByExamIdAndSubjectiveScoreListNotNull(Integer examId);

    public List<ExamStudent> findByExamIdAndSubjectCodeAndUploadTimeNotNull(int examId, String code);

    @Query("select s from ExamStudent s where s.examId=?1 and s.subjectCode=?2 and s.absent=false and s.uploadTime!=null "
            + "and not exists (select l.id from MarkLibrary l where l.studentId=s.id and l.groupNumber=?3)")
    public List<ExamStudent> findUnLibraryStudent(int examId, String subjectCode, int groupNumber, Pageable page);

    @Query("select s from ExamStudent s where s.examId=?1 and s.subjectCode=?2 and s.absent=false and s.uploadTime!=null and s.uploadTime>=?4 "
            + "and not exists (select l.id from MarkLibrary l where l.studentId=s.id and l.groupNumber=?3)")
    public List<ExamStudent> findUnLibraryStudent(int examId, String subjectCode, int groupNumber, Date minUploadTime,
            Pageable page);

    @Query("select s.id from ExamStudent s where s.examId=?1 and s.subjectCode=?2 and s.absent=true and "
            + "exists (select l.id from MarkLibrary l where l.studentId=s.id)")
    public List<Integer> findAbsentLibraryStudent(int examId, String subjectCode);
    
    @Query("select sum(case when (s.objectiveScore + s.subjectiveScore) between 0 and 49 then 1 else 0 end),"
    		+ "sum(case when (s.objectiveScore + s.subjectiveScore) between 50 and 59 then 1 else 0 end) ,"
    		+ "sum(case when (s.objectiveScore + s.subjectiveScore) between 60 and 69 then 1 else 0 end) ,"
    		+ "sum(case when (s.objectiveScore + s.subjectiveScore) between 70 and 79 then 1 else 0 end) ,"
    		+ "sum(case when (s.objectiveScore + s.subjectiveScore) between 80 and 89 then 1 else 0 end) ,"
    		+ "sum(case when (s.objectiveScore + s.subjectiveScore) between 90 and 100 then 1 else 0 end) "
    		+ " from ExamStudent s where s.examId=?1 and s.subjectCode=?2 and s.upload=?3  and s.absent=?4 ")
	public List<Object[]> statisticsByExamIdAndSubjectCode(Integer examId,
			String code, Boolean upload, boolean absent);

    @Query("select sum(case when (s.objectiveScore + s.subjectiveScore) between 0 and 49 then 1 else 0 end),"
    		+ "sum(case when (s.objectiveScore + s.subjectiveScore) between 50 and 59 then 1 else 0 end) ,"
    		+ "sum(case when (s.objectiveScore + s.subjectiveScore) between 60 and 69 then 1 else 0 end) ,"
    		+ "sum(case when (s.objectiveScore + s.subjectiveScore) between 70 and 79 then 1 else 0 end) ,"
    		+ "sum(case when (s.objectiveScore + s.subjectiveScore) between 80 and 89 then 1 else 0 end) ,"
    		+ "sum(case when (s.objectiveScore + s.subjectiveScore) between 90 and 100 then 1 else 0 end) "
    		+ " from ExamStudent s where s.examId=?1 and s.subjectCode=?2 ")
	public List<Object[]> statisticsByExamIdAndSubjectCode(Integer examId,
			String code);

}
